(function(operationStringified, settings) {

  let operation, banner, offers;

  try {
    operation = JSON.parse(operationStringified);
    if (operation.banner.enabled) {
      banner = operation.banner;
      offers = operation.offers;
    } else return ;
  } catch (error) {
    return console.error('Kastor Studio: There is an error in the ScriptTag, please contact us on https://kastorstudio.crisp.help/ or at support@kastor.studio and indicates the following error:\n\n', error);
  }

  function getCookie(name) {
    let cookieString = RegExp(name+"=[^;]+").exec(document.cookie);
    return decodeURIComponent(!!cookieString ? cookieString.toString().replace(/^[^=]+./,"") : "");
  }

  function setCookie(name, value, days) {
    const d = new Date();
    d.setTime(d.getTime() + (days * 24 * 60 * 60 * 1000));
    const expires = "expires="+ d.toUTCString();
    document.cookie = name + "=" + value + ";" + expires + ";path=/";
  }

  function uuidv4() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
      const r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
      return v.toString(16);
    });
  }

  try {
    // Create stylesheet
    const styles = `
      @import url('https://fonts.googleapis.com/css?family=Montserrat:100,200,300,400&display=swap');

      #ksm-root {
        width: 100%;
        z-index: 2147483647;
      }

      /*
      ** Banner
      */
      #ksm-banner {
        position: relative;
        width: 100%;
        padding: 12px;
        text-align: center;
        letter-spacing: 2px;
        cursor: pointer;
        z-index: 2147483644;
      }
      .ksm-banner--eclipsed {
        font-size: ${banner.styles.fontSize || '14px'} !important;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
        font-weight: ${banner.styles.fontWeight || 'normal'} !important;
        text-transform: ${banner.styles.textTransform || 'normal'} !important;
        background: ${banner.styles.backgroundColor || 'black'} !important;
        color: ${banner.styles.color || 'white'} !important;
      }
      .ksm-banner--expanded {
        font-size: ${banner.expanded.styles.fontSize || '14px'} !important;
        font-family: ${banner.expanded.styles.fontFamily || 'Montserrat'} !important;
        font-weight: ${banner.expanded.styles.fontWeight || 'normal'} !important;
        text-transform: ${banner.expanded.styles.textTransform || 'normal'} !important;
        background: ${banner.expanded.styles.backgroundColor || 'black'} !important;
        color: ${banner.expanded.styles.color || 'white'} !important;
      }
      #ksm-banner__collapse {
        position: absolute;
        width: 100%;
        max-height: 0px;
        text-align: center;
        overflow: auto;
        white-space: nowrap;
        background-color: white;
        transition: max-height 0.3s linear;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
        box-shadow: 0px 5px 15px rgba(0, 0, 0, .1);
        z-index: 2147483644;
      }

      /*
      ** Offer Card
      */
      .ksm-offer_card {
        background-color: white;
        display: inline-block;
        border: 1px solid #ededed;
        width: 100%;
        max-width: 340px;
        height: 255px;
        margin: 20px 10px;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
      }
      .ksm-offer_card__image {
        width: 100%;
        height: 75%;
        object-fit: cover;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
      }
      .ksm-offer_card__bottom {
        display: flex;
        justify-content: space-between;
        background-color: white;
        padding: 10px 12px;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
      }
      .ksm-offer_card__bottom__btn-primary {
        border: 1px solid #858585;
        border-radius: ${banner.buttons.styles.borderRadius || '0px'}
        text-transform: uppercase;
        padding: 10px;
        background-color: white;
        font-size: 12px;
        color: black;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
      }
      .ksm-offer_card__bottom__btn-primary:hover {
        cursor: pointer;
        background-color: black;
        border: 1px solid white;
        color: white;
      }
      .ksm-offer_card__bottom__btn-second {
        border: 1px solid white;
        border-radius: ${banner.buttons.styles.borderRadius || '0px'}
        text-transform: uppercase;
        padding: 10px;
        background-color: white;
        font-size: 12px;
        color: black;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
      }
      .ksm-offer_card__bottom__btn-second:hover {
        cursor: pointer;
        border: 1px solid #f0f0f0;
        background-color: #f0f0f0;
      }

      /*
      ** Modal
      */
      #ksm-modal {
        display: none;
        position: fixed;
        margin: auto;
        top: 0;
        left: 0;
        bottom: 0;
        right: 0;
        width: 60%;
        height: 60%;
        background-color: white;
        box-shadow: 0px 4px 18px rgba(0, 0, 0, .1);
        padding: 32px;
        font-family: ${banner.styles.fontFamily || 'Montserrat'} !important;
        overflow: auto;
        z-index: 2147483646;
      }
      #ksm-modal__close {
        position: absolute;
        right: 16px;
        top: 16px;
        width: 32px;
        height: 32px;
        opacity: 0.3;
        cursor: pointer;
      }
      #ksm-modal__close:hover {
        opacity: 1;
      }
      #ksm-modal__close:before, #ksm-modal__close:after {
        position: absolute;
        left: 15px;
        content: ' ';
        height: 33px;
        width: 2px;
        background-color: #333;
      }
      #ksm-modal__close:before {
        transform: rotate(45deg);
      }
      #ksm-modal__close:after {
        transform: rotate(-45deg);
      }
      #ksm-modal__title {
        font-size: 22px;
        font-weight: bold;
      }
      #ksm-modal__html {
        margin-top: 20px;
      }

      /*
      ** Overlay
      */
      #ksm-overlay {
        display: none;
        background: #000;
        opacity: 0.3;
        position: fixed;
        top: 0;
        left: 0;
        width: 100%;
        height: 100%;
        z-index: 2147483643;
      }
    `;

    // Insert stylesheet
    const stylesheet = document.createElement('style');
    stylesheet.type = 'text/css';
    stylesheet.innerText = styles;
    document.head.appendChild(stylesheet);

    // Create App Root HTML structure
    const html = `
      <div id="ksm-banner" class="ksm-banner--eclipsed">
        <div id="ksm-banner__title">${banner.title}</div> 
        <div id="ksm-banner__countdown"></div>
      </div>
      <div id="ksm-banner__collapse"></div>
      <div id="ksm-modal">
        <div id="ksm-modal__title"></div>
        <div id="ksm-modal__html"></div>
        <div id="ksm-modal__close"></div>
      </div>
      <div id="ksm-overlay"></div>
    `;

    // Insert App Root HTML structure
    const root = document.createElement('div');
    root.setAttribute('id', 'ksm-root');
    root.innerHTML = html;
    document.body.insertBefore(root, document.body.firstChild);

    // Vars
    const bannerWrap     = document.getElementById('ksm-banner');
    const bannerTitle    = document.getElementById('ksm-banner__title');
    const bannerCollapse = document.getElementById('ksm-banner__collapse');
    const countdown      = document.getElementById('ksm-banner__countdown');
    const modalClose     = document.getElementById('ksm-modal__close');
    const modal          = document.getElementById('ksm-modal');
    const modalTitle     = document.getElementById('ksm-modal__title');
    const modalHTML      = document.getElementById('ksm-modal__html');
    const overlay        = document.getElementById('ksm-overlay');
    const appRoot        = document.getElementById('ksm-root');

    // Functions
    function insertOffer(offer, index) {
      const offerHTML = `
        <div class="ksm-offer_card">
          <img class="ksm-offer_card__image" src="${offer.image}" alt="${offer.title}" onerror="this.onerror=null;this.src='https://kastor-sales-manager.s3-eu-west-1.amazonaws.com/kastor_studio_placeholder_offer.png';" />
          <div class="ksm-offer_card__bottom">
            <div class="ksm-offer_card__bottom__btn-primary" ksm-offer-index="${index}">${banner.buttons.content.link || "Apply"}</div>
            <div class="ksm-offer_card__bottom__btn-second" ksm-offer-index="${index}">${banner.buttons.content.popup || "Details"}</div>
          </div>
        </div>
      `
      bannerCollapse.innerHTML += offerHTML;
    }

    function handleModal(title, html) {
      modalTitle.innerHTML  = title;
      modalHTML.innerHTML   = html;
      if (modal.style.display !== 'block') {
        modal.style.display = 'block';
        overlay.style.zIndex = '2147483645';
      } else {
        modal.style.display = 'none';
        overlay.style.zIndex = '2147483643';
      }
    }
    function handleBanner() {
      if (offers.length === 0) return;
      handleOverlay();
      bannerCollapse.style.maxHeight !== '500px' ?
        bannerCollapse.style.maxHeight = '500px' : bannerCollapse.style.maxHeight = '0px';
      if (bannerCollapse.style.maxHeight === '500px') {
        bannerWrap.classList.add('ksm-banner--expanded');
        bannerWrap.classList.remove('ksm-banner--eclipsed');
        bannerTitle.innerHTML = banner.expanded.title || banner.title;
      } else {
        bannerWrap.classList.add('ksm-banner--eclipsed');
        bannerWrap.classList.remove('ksm-banner--expanded');
        bannerTitle.innerHTML = banner.title;
      }
    }
    function handleOverlay() {
      overlay.style.display !== 'block' ?
        overlay.style.display = 'block' : overlay.style.display = 'none';
    }

    function handlePrimaryButton(event) {
      const offer = offers[event.target.getAttribute('ksm-offer-index')];
      window.location = `https://${operation.shop}/discount/${offer.discount_code}?redirect=${offer.link}`;
    }
    function handleSecondButton(event) {
      const offer = offers[event.target.getAttribute('ksm-offer-index')];
      handleModal(offer.popup.title, offer.popup.html);
    }

    // Insert all offers from stringified settings
    if (offers) offers.forEach(function(offer, index) { insertOffer(offer, index) });

    // Set onclick actions
    Array.from(document.getElementsByClassName('ksm-offer_card__bottom__btn-primary')).forEach(
      function(button) { button.onclick = handlePrimaryButton }
    );
    Array.from(document.getElementsByClassName('ksm-offer_card__bottom__btn-second')).forEach(
      function(button) { button.onclick = handleSecondButton }
    );

    bannerWrap.onclick = handleBanner;
    modalClose.onclick = handleModal;

    // Event listeners
    window.addEventListener('click', function(e) {
      const isInButton = e.target.classList.contains('ksm-offer_card__bottom__btn-second');
      const modalDisplay = modal.style.display;
      if (!modal.contains(e.target) && !isInButton && modalDisplay === 'block')
        handleModal();
      if (modalDisplay !== 'block' && e.target.id === 'ksm-overlay')
        handleBanner();
    });
    window.addEventListener('scroll', function(e) {
      const modalDisplay = modal.style.display;
      if (modalDisplay !== 'block' && bannerCollapse.style.maxHeight === '500px') {
        handleBanner();
        overlay.style.display = 'none';
      }
    });

    // Countdown
    // if (banner.countdown) {
    //   const countdownDate = new Date(banner.countdown).getTime();
    //   const countdownTimer = setInterval(function() {
    //     const now = new Date().getTime();
    //     const interval = countdownDate - now;
  
    //     const days = Math.floor(interval / (1000 * 60 * 60 * 24));
    //     const hours = Math.floor((interval % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60));
    //     const minutes = Math.floor((interval % (1000 * 60 * 60)) / (1000 * 60));
    //     const seconds = Math.floor((interval % (1000 * 60)) / 1000);
  
    //     countdown.innerHTML = days + 'd ' + hours + 'h ' + minutes + 'm ' + seconds + 's ';
  
    //     if (interval < 0) {
    //       clearInterval(countdownTimer);
    //       countdown.innerHTML = '';
    //     }
    //   }, 1000);
    // }

    // Handle Cookie _kastorid
    // => gaClientId or uuidv4
    // let cookieId = getCookie('_kastorid');
    // if (!cookieId) {
    //   if (!ga)
    //     return setCookie('_kastorid', uuidv4(), 30);

    //   ga(function(tracker) {
    //     const clientId = tracker.get('clientId');
    //     if (clientId) {
    //       setCookie('_kastorid', clientId, 30);
    //       cookieId = clientId;
    //     }
    //   });

    //   if (!cookieId)
    //     return setCookie('_kastorid', uuidv4(), 30);
    // }

    // Analytics
    // const gaCategory = 'kastor-studio_sales-manager';
    // const setGA = {
    //   click: function(element, label) {
    //     function trigger() {
    //       if (!ga) return;
    //       ga('send', 'event', gaCategory, 'click', label);

    //       const endpoint = `https://fn5cyo4a5i.execute-api.eu-west-1.amazonaws.com/dev/analytics`;
    //       const params = `category=${gaCategory}&label=${label}&action=click&id=${cookieId}`;
    //       const xhr = new XMLHttpRequest();
    //       xhr.open('GET', endpoint + '?' + params, true);
    //       // xhr.onreadystatechange = function() { return xhr.readyState === 4 ? console.log(xhr.response) : null };
    //       xhr.send(null);
    //     }
    //     element.addEventListener('click', trigger);
    //   }
    // }

    // function setAnalytics() {
    //   if (!ga) return;
    //   setGA.click(bannerWrap, 'banner');
    //   Array.from(document.getElementsByClassName('ksm-offer_card__bottom__btn-primary')).forEach(
    //     function(button, index) { setGA.click(button, `button-primary-${index}`) }
    //   );
    //   Array.from(document.getElementsByClassName('ksm-offer_card__bottom__btn-second')).forEach(
    //     function(button, index) { setGA.click(button, `button-second-${index}`) }
    //   );
    // }

    // setAnalytics();

  } catch (error) {
    return console.error('Kastor Studio: There is an error in the ScriptTag, please contact us on https://kastorstudio.crisp.help/ or at support@kastor.studio and indicates the following error:\n\n', error);
  }
})
()